<?php
/**
 * [PHPFOX_HEADER]
 */

defined('PHPFOX') or exit('NO DICE!');

/**
 * 
 * 
 * @copyright		[PHPFOX_COPYRIGHT]
 * @author  		Raymond Benc
 * @package 		Phpfox_Service
 * @version 		$Id: callback.class.php 3163 2011-09-21 11:46:40Z Miguel_Espinoza $
 */
class Custom_Service_Callback extends Phpfox_Service 
{
	/**
	 * Class constructor
	 */	
	public function __construct()
	{	
		
	}
	
	public function exportModule($sProduct, $sModule)
	{
		return Phpfox::getService('custom')->export($sProduct, $sModule);
	}
	
	public function reparserList()
	{
		$aFields = $this->database()->select('*')
			->from(Phpfox::getT('custom_field'))
			->where('var_type IN(\'textarea\', \'text\')')
			->execute('getRows');
		
		$aCallback = array();
		foreach ($aFields as $aField)
		{
			if (!Phpfox::isModule($aField['module_id']))
			{
				continue;
			}
			
			$aCallback[] = array(
				'name' => Phpfox::getPhrase($aField['phrase_var_name']) . ' (' . Phpfox::getPhrase('admincp.custom_field') . ')',
				'table' => array('user_custom_value', 'user_custom'),
				'original' => 'cf_' . $aField['field_name'],
				'parsed' => 'cf_' . $aField['field_name'],
				'item_field' => 'user_id'
			);
		}	
		
		return $aCallback;
	}
	
	
	public function addLike($iItemId, $bDoNotSendEmail = false)
	{
		
		$aRow = $this->database()->select('field_id, phrase_var_name')
			->from(Phpfox::getT('custom_field'))
			->where('field_id = ' . (int) $iItemId)
			->execute('getSlaveRow');		
		
		if (!isset($aRow['field_id']))
		{
			return false;
		}
		
		$aFeed = $this->database()->select('f.*, u.user_name')
				->from(Phpfox::getT('feed'), 'f')
				->where('feed_id = ' . (int)Phpfox::getLib('request')->get('parent_id'))
				->join(Phpfox::getT('user'), 'u', 'u.user_id = f.user_id')
				->execute('getSlaveRow');
		
		/* Check if the field exists for the total_like*/
		$iExists = $this->database()->select('field_id')
				->from(Phpfox::getT('user_custom_data'))
				->where('field_id = ' . $iItemId . ' AND user_id = ' . $aFeed['user_id'])
				->execute('getSlaveField');
		if ($iExists > 0)
		{
			$this->database()->updateCount('like', 'type_id = \'custom\' AND item_id = ' . (int) $iItemId . '', 'total_like', 'user_custom_data', 'field_id = ' . (int) $iItemId . ' AND user_id = ' . $aFeed['user_id']);	
		}
		else
		{
			$this->database()->insert(Phpfox::getT('user_custom_data'), array(
				'user_id' => $aFeed['user_id'],
				'field_id' => $iItemId,
				'total_like' => 1));
		}
		
		if (!$bDoNotSendEmail)
		{
			$sLink = Phpfox::getLib('url')->makeUrl($aFeed['user_name'], array('feed-id' => $aFeed['feed_id']));

			Phpfox::getLib('mail')->to($aFeed['user_id'])
					->subject(Phpfox::getUserBy('full_name') . " liked your change on \"" . Phpfox::getPhrase($aRow['phrase_var_name']) . "\"")
					->message(Phpfox::getUserBy('full_name') . " liked your change on \"<a href=\"" . $sLink . "\">" . Phpfox::getPhrase($aRow['phrase_var_name']) . "</a>\"\nTo view this update follow the link below:\n<a href=\"" . $sLink . "\">" . $sLink . "</a>")
					->send();

			Phpfox::getService('notification.process')->add('custom_like', $aFeed['feed_id'], $aFeed['user_id']);
		}
	}

	public function deleteLike($iItemId)
	{
		$this->database()->updateCount('like', 'type_id = \'custom\' AND item_id = ' . (int) $iItemId . '', 'total_like', 'user_custom_data', 'field_id = ' . (int) $iItemId . ' AND user_id = ' . Phpfox::getUserId());
		
	}

	public function getNotificationLike($aNotification)
	{
		$aRow = $this->database()->select('b.field_id, b.phrase_var_name, owner.user_name, owner.user_id, owner.gender, owner.full_name')
				->from(Phpfox::getT('custom_field'), 'b')
			->join(Phpfox::getT('user'),'owner', 'owner.user_id = ' . (int)$aNotification['item_user_id'])
				->where('b.field_id = ' . (int) $aNotification['item_id'])
				->execute('getSlaveRow');
		
		if (!isset($aRow['user_id']))
		{
			return false;
		}		

		$sPhrase = '';
		if ($aNotification['user_id'] == $aRow['user_id'])
		{
			$sPhrase = Phpfox::getService('notification')->getUsers($aNotification) . ' liked ' . Phpfox::getService('user')->gender($aRow['gender'], 1) . ' own profile update "' . Phpfox::getLib('parse.output')->shorten(Phpfox::getPhrase($aRow['phrase_var_name']), Phpfox::getParam('notification.total_notification_title_length'), '...') . '"';
		}
		elseif ($aRow['user_id'] == Phpfox::getUserId())
		{
			$sPhrase = Phpfox::getService('notification')->getUsers($aNotification) . ' liked your profile update "' . Phpfox::getLib('parse.output')->shorten(Phpfox::getPhrase($aRow['phrase_var_name']), Phpfox::getParam('notification.total_notification_title_length'), '...') . '"';
		}
		else
		{
			$sPhrase = Phpfox::getService('notification')->getUsers($aNotification) . ' liked <span class="drop_data_user">' . $aRow['full_name'] . '\'s</span> profile update "' . Phpfox::getLib('parse.output')->shorten(Phpfox::getPhrase($aRow['phrase_var_name']), Phpfox::getParam('notification.total_notification_title_length'), '...') . '"';
		}

		return array(
			'link' => Phpfox::getLib('url')->makeUrl($aRow['user_name']),//Phpfox::getLib('url')->permalink('custom', $aRow['blog_id'], $aRow['title']),
			'message' => $sPhrase,
			'icon' => Phpfox::getLib('template')->getStyle('image', 'activity.png', 'blog')
		);
	}

	/**
	 * This function passes the params to output an entry in the feed for when
	 * a user has changed the status of their relationship
	 * @param type $aFeed 
	 */
	public function getActivityFeedRelation($aFeed)
	{
		if ($aFeed['type_id'] == 'custom_relation' && $aFeed['user_id'] != Phpfox::getUserId())
		{
		    //return false;
		}
		/* New status */
		$aRelation = $this->database()->select('crd.total_like, crd.total_comment, u.gender, cr.relation_id, 
				crd.status_id, crd.relation_data_id, cr.phrase_var_name, 
				u.user_id, u.full_name, u.user_name, 
				u2.user_id AS with_user_id, 
				u2.full_name as with_full_name, 
				u2.user_name as with_user_name')
				->from(Phpfox::getT('custom_relation_data'), 'crd')
				->join(Phpfox::getT('custom_relation'), 'cr', 'cr.relation_id = crd.relation_id')
				->leftJoin(Phpfox::getT('user'), 'u', 'u.user_id = crd.user_id')
				->leftJoin(Phpfox::getT('user'), 'u2', 'u2.user_id = crd.with_user_id')
				->where('crd.relation_data_id = ' . (int) $aFeed['item_id'])
				->execute('getSlaveRow');
		
		if (!isset($aRelation['relation_data_id']) || empty($aRelation['relation_data_id']))
		{
			return false;
		}	
		$aRelation['relation_with'] = ($aRelation['user_id'] == $aFeed['user_id']) ? 0 : 1;
		/* previous status */
		$sPrevious = $this->database()->select('cr.phrase_var_name')
				->from(Phpfox::getT('custom_relation'), 'cr')
				->join(Phpfox::getT('custom_relation_data'), 'crd', 'crd.relation_id = cr.relation_id')
				->where('crd.user_id = ' . $aFeed['user_id'] . ' AND crd.relation_data_id < ' . $aRelation['relation_data_id'] . '')
				->order('crd.relation_data_id DESC')
				->limit(1)
				->execute('getSlaveField');
		
		/* If this status is blank and the user did not have any other status before we 
		 * should not display this message as it happened when updating other custom field
		 */
		if ($sPrevious == '')
		{
			$iPrevious = $this->database()->select('relation_id')
					->from(Phpfox::getT('custom_relation_data'))
					->where('relation_data_id = ' . (int)$aFeed['item_id'])
					->execute('getSlaveField');
			if ($iPrevious == 1)
			{
				return false;
			}
		}
		
		/* Replacements for the phrase */
		$aReplace = array(
			'full_name' => $aRelation['full_name'],
			'user_name' => $aRelation['user_name'],
			'their' => Phpfox::getService('user')->gender($aRelation['gender'], 1),
			'sOldStatus' => Phpfox::getPhrase($sPrevious),
			'sNewStatus' => Phpfox::getPhrase($aRelation['phrase_var_name'])
		);
		
		/* we only need the previous state if the current phrase has {previous_status} in its text */
		//d($aRelation);die();
		$sPhrase = Phpfox::getService('custom')->getRelationshipPhrase($aRelation, $aFeed, $aReplace, $sPrevious);
		/* For now lets just send the user to the friend page */
		$sLink = Phpfox::getLib('url')->makeUrl('friend');

		$aReturn = array(
			'feed_link' => $sLink,
			'feed_title' => '',
			'feed_info' => $sPhrase,
			'total_comment' => $aRelation['total_comment'],
			'feed_total_like' => $aRelation['total_like'],
			'feed_is_liked' => isset($aRow['is_liked']) ? $aRow['is_liked'] : false,
			'feed_icon' => Phpfox::getLib('image.helper')->display(array('theme' => 'misc/heart.png', 'return_url' => true)),
			'time_stamp' => $aFeed['time_stamp'],
			'enable_like' => true,
			'like_type_id' => 'custom_relation',
			'comment_type_id' => 'custom_relation'
		);
		//die();
		return $aReturn;
	}

	public function addCommentRelation($aVals, $iUserId = null, $sUserName = null)
	{
		$aRow = $this->database()->select('cr.phrase_var_name, cf.relation_data_id, f.feed_id, u.full_name, u.gender, u.user_id, u.user_name')
				->from(Phpfox::getT('custom_relation_data'), 'cf')
				->join(Phpfox::getT('custom_relation'), 'cr', 'cr.relation_id = cf.relation_id')
				->join(Phpfox::getT('feed'), 'f', 'f.feed_id = ' . $aVals['is_via_feed'])
				->join(Phpfox::getT('user'), 'u', 'u.user_id = f.user_id')
				->where('cf.relation_data_id = ' . (int) $aVals['item_id'])
				->execute('getSlaveRow');
		
		if (!isset($aRow['relation_data_id']))
		{
			return Phpfox_Error::trigger('Invalid callback on comment.');
		}

		// Update the post counter if its not a comment put under moderation or if the person posting the comment is the owner of the item.
		//$this->database()->updateCounter('user_custom_data', 'total_comment', 'field_id', $aRow['field_id'] . ' AND user_id = ' . $aRow['user_id']);				
		$iCount = $this->database()->select('total_comment')
				->from(Phpfox::getT('custom_relation_data'))
				->where('relation_data_id = ' . (int) $aRow['relation_data_id'])
				->execute('getSlaveField');

		$this->database()->update(Phpfox::getT('custom_relation_data'), array('total_comment' => ($iCount + 1)), 'relation_data_id = ' . (int) $aRow['relation_data_id']);

		if (Phpfox::isModule('feed'))
		{
			Phpfox::getService('feed.process')->add('comment_relation', $aRow['relation_data_id']);
		}
		// Send the user an email
		$sLink = Phpfox::getLib('url')->makeUrl($aRow['user_name'], array('feed-id' => $aRow['feed_id'])); //Phpfox::getLib('url')->permalink('custom', $aRow['field_id'], Phpfox::getPhrase($aRow['phrase_var_name']));

		Phpfox::getService('comment.process')->notify(array(
			'user_id' => $aRow['user_id'],
			'item_id' => $aRow['relation_data_id'],
			'owner_subject' => Phpfox::getUserBy('full_name') . ' commented on your profile update "' . $this->preParse()->clean(Phpfox::getPhrase($aRow['phrase_var_name']), 100) . '".',
			'owner_message' => Phpfox::getUserBy('full_name') . " commented on your profile_update \"<a href=\"" . $sLink . "\">" . Phpfox::getPhrase($aRow['phrase_var_name']) . "</a>\".\nTo see the comment thread, follow the link below:\n<a href=\"" . $sLink . "\">" . $sLink . "</a>",
			'owner_notification' => 'comment.add_new_comment',
			'notify_id' => 'custom_comment_relation',
			'mass_id' => 'custom',
			'mass_subject' => (Phpfox::getUserId() == $aRow['user_id'] ? Phpfox::getUserBy('full_name') . ' commented on ' . Phpfox::getService('user')->gender($aRow['gender'], 1) . ' profile update.' : Phpfox::getUserBy('full_name') . ' commented on ' . $aRow['full_name'] . '\'s video.'),
			'mass_message' => (Phpfox::getUserId() == $aRow['user_id'] ? Phpfox::getUserBy('full_name') . " commented on " . Phpfox::getService('user')->gender($aRow['gender'], 1) . " profile update \"<a href=\"" . $sLink . "\">" . Phpfox::getPhrase($aRow['phrase_var_name']) . "</a>\".\nTo see the comment thread, follow the link below:\n<a href=\"" . $sLink . "\">" . $sLink . "</a>" : Phpfox::getUserBy('full_name') . " commented on " . $aRow['full_name'] . "'s profile update \"<a href=\"" . $sLink . "\">" . Phpfox::getPhrase($aRow['phrase_var_name']) . "</a>\".\nTo see the comment thread, follow the link below:\n<a href=\"" . $sLink . "\">" . $sLink . "</a>")
				)
		);
	}

	public function addLikeRelation($iItemId, $bDoNotSendEmail = false)
	{
		$aRow = $this->database()->select('relation_data_id, total_like, user_id')
				->from(Phpfox::getT('custom_relation_data'))
				->where('relation_data_id = ' . (int) $iItemId)
				->execute('getSlaveRow');

		$aRow['title'] = 'Something something here';
		if (!isset($aRow['relation_data_id']))
		{
			return false;
		}

		$this->database()->updateCount('like', 'type_id = \'custom_relation\' AND item_id = ' . (int) $iItemId . '', 'total_like', 'custom_relation_data', 'relation_data_id = ' . (int) $iItemId);

		if (!$bDoNotSendEmail)
		{
			$sLink = Phpfox::getLib('url')->makeUrl('friend'); //Phpfox::permalink('friend');

			Phpfox::getLib('mail')->to($aRow['user_id'])
					->subject(Phpfox::getUserBy('full_name') . " liked your change in relationship \"" . $aRow['title'] . "\"")
					->message(Phpfox::getUserBy('full_name') . " liked your change in relationship \"<a href=\"" . $sLink . "\">" . $aRow['title'] . "</a>\"\nTo view this follow the link below:\n<a href=\"" . $sLink . "\">" . $sLink . "</a>")
					->send();

			Phpfox::getService('notification.process')->add('custom_relation_like', $aRow['relation_data_id'], $aRow['user_id']);
		}
	}

	public function deleteLikeRelation($iItemId)
	{
		$this->database()->updateCount('like', 'type_id = \'custom_relation\' AND item_id = ' . (int) $iItemId . '', 'total_like', 'custom_relation_data', 'relation_data_id = ' . (int) $iItemId);
	}

	
	/**
	 * The return of this function eventually goes to addComment
	 * @param type $iId
	 * @return string 
	 */
	public function getCommentItemRelation($iId)
	{		
		$aRow = $this->database()->select('relation_data_id AS comment_item_id, f.user_id AS comment_user_id')
				->join(Phpfox::getT('feed'), 'f', 'f.item_id = cf.relation_data_id AND f.type_id = "custom_relation"')
				->from(Phpfox::getT('custom_relation_data'), 'cf')
				->where('cf.relation_data_id = ' . (int) $iId)
				->execute('getSlaveRow');

		$aRow['privacy_comment'] = '0';
		$aRow['comment_view_id'] = '0';

		if (!Phpfox::getService('comment')->canPostComment($aRow['comment_user_id'], $aRow['privacy_comment']))
		{
			Phpfox_Error::set('Unable to post a comment on this item due to privacy settings.');

			unset($aRow['comment_item_id']);
		}

		return $aRow;
	}
	
	public function getNotificationComment_Relation($aNotification)
	{
		
		$aRow = $this->database()->select('l.relation_data_id, l.relation_data_id, cr.phrase_var_name, u.user_id, u.gender, u.user_name, u.full_name')	
			->from(Phpfox::getT('custom_relation_data'), 'l')
			->join(Phpfox::getT('custom_relation'), 'cr', 'cr.relation_id = l.relation_id')
			->join(Phpfox::getT('user'), 'u', 'u.user_id = l.user_id')
			->where('l.relation_data_id = ' . (int) $aNotification['item_id'])
			->execute('getSlaveRow');
		
		$aRow['name'] = Phpfox::getPhrase($aRow['phrase_var_name']);
		
		if (!isset($aRow['relation_data_id']))
		{
			return false;
		}
		/* get the feed id */
		$iFeed = $this->database()->select('feed_id')
				->from(Phpfox::getT('feed'))
				->where('type_id = "custom_relation" AND item_id = ' . $aRow['relation_data_id'])
				->execute('getSlaveField');
		
		$sPhrase = '';
		if ($aNotification['user_id'] == $aRow['user_id'] && !isset($aNotification['extra_users']))
		{
			$sPhrase = Phpfox::getService('notification')->getUsers($aNotification) . ' commented on ' . Phpfox::getService('user')->gender($aRow['gender'], 1) . ' album "' . Phpfox::getLib('parse.output')->shorten($aRow['name'], Phpfox::getParam('notification.total_notification_title_length'), '...') . '"';	
		}
		elseif ($aRow['user_id'] == Phpfox::getUserId())
		{
			$sPhrase = Phpfox::getService('notification')->getUsers($aNotification) . ' commented on your relationship status "' . Phpfox::getLib('parse.output')->shorten($aRow['name'], Phpfox::getParam('notification.total_notification_title_length'), '...') . '"';
		}
		else 
		{
			$sPhrase = Phpfox::getService('notification')->getUsers($aNotification) . ' commented on <span class="drop_data_user">' . $aRow['full_name'] . '\'s</span> relationship status "' . Phpfox::getLib('parse.output')->shorten($aRow['name'], Phpfox::getParam('notification.total_notification_title_length'), '...') . '"';
		}
			
		return array(
			'link' => Phpfox::getLib('url')->permalink($aRow['user_name'], 'feed_'. $iFeed),
			'message' => $sPhrase,
			'icon' => Phpfox::getLib('template')->getStyle('image', 'activity.png', 'blog')
		);
	}
	
	public function getNotificationRelation_Like($aNotification)
	{
		$aRow = $this->database()->select('ms.relation_data_id, cr.phrase_var_name, ms.user_id, u.gender, u.full_name, u.user_name')	
			->from(Phpfox::getT('custom_relation_data'), 'ms')
			->join(Phpfox::getT('custom_relation'), 'cr', 'cr.relation_id = ms.relation_id')
			->join(Phpfox::getT('user'), 'u', 'u.user_id = ms.user_id')
			->where('ms.relation_data_id = ' . (int) $aNotification['item_id'])
			->execute('getSlaveRow');
		
		$aRow['name'] = Phpfox::getPhrase($aRow['phrase_var_name']);
		
		if (!isset($aRow['relation_data_id']))
		{
			return false;
		}			
		
		/* get the feed id */
		$iFeed = $this->database()->select('feed_id')
				->from(Phpfox::getT('feed'))
				->where('type_id = "custom_relation" AND item_id = ' . $aRow['relation_data_id'])
				->execute('getSlaveField');
		
		$sPhrase = '';
		if ($aNotification['user_id'] == $aRow['user_id'])
		{
			$sPhrase = Phpfox::getService('notification')->getUsers($aNotification) . ' liked ' . Phpfox::getService('user')->gender($aRow['gender'], 1) . ' own relationship status "' . Phpfox::getLib('parse.output')->shorten($aRow['name'], Phpfox::getParam('notification.total_notification_title_length'), '...') . '"';	
		}
		elseif ($aRow['user_id'] == Phpfox::getUserId())		
		{
			$sPhrase = Phpfox::getService('notification')->getUsers($aNotification) . ' liked your relationship status "' . Phpfox::getLib('parse.output')->shorten($aRow['name'], Phpfox::getParam('notification.total_notification_title_length'), '...') . '"';
		}
		else 
		{
			$sPhrase = Phpfox::getService('notification')->getUsers($aNotification) . ' liked <span class="drop_data_user">' . $aRow['full_name'] . '\'s</span> relationship status "' . Phpfox::getLib('parse.output')->shorten($aRow['name'], Phpfox::getParam('notification.total_notification_title_length'), '...') . '"';
		}
			
		return array(
			'link' => Phpfox::getLib('url')->permalink($aRow['user_name'], 'feed_' . $iFeed),
			'message' => $sPhrase,
			'icon' => Phpfox::getLib('template')->getStyle('image', 'activity.png', 'blog')
		);	
	}
	
	
	public function getActivityFeed($aItem)
	{
		$sLink = Phpfox::getLib('url')->makeUrl($aItem['user_name']);
		$aReturn = array(
			'feed_link' => $sLink,
			'feed_title' => '',
			'feed_info' => 'updated ' . Phpfox::getService('user')->gender($aItem['gender'], 1) . ' profile information',
			//'total_comment' => $aRow['total_comment'],
			//'feed_total_like' => $aRow['total_like'],
			//'feed_is_liked' => isset($aRow['is_liked']) ? $aRow['is_liked'] : false,
			'feed_icon' => Phpfox::getLib('image.helper')->display(array('theme' => 'misc/page_edit.png', 'return_url' => true)),
			'time_stamp' => $aItem['time_stamp'],
			'enable_like' => false,
			//'comment_type_id' => 'custom',
			//'like_type_id' => 'custom'
		);

		return $aReturn;
		
	}
	public function getAjaxCommentVar()
	{
		return null;
	}

	public function getAjaxCommentVarRelation()
	{
		return null;
	}
	public function updateCounterList()
	{
		$aList = array();		
		
		$aList[] =	array(
			'name' => 'Import Custom Fields',
			'id' => 'import-custom-fields'			
		);			

		return $aList;
	}
	public function updateCounter($iId, $iPage = null, $iPageLimit = null)
	{
	    if ($iId == 'import-custom-fields')
	    {		
		// get all the custom fields
		$aFields = $this->database()->select('*')
			->from(Phpfox::getT('custom_field'))
			->execute('getSlaveRows');
		$sWhere = '';
		foreach ($aFields as $aField)
		{
		    $sWhere .= '(component = "cf_' . $aField['field_name'] .'" AND module_id="' . $aField['module_id'] .'") OR ';
		}
		$sWhere = rtrim($sWhere, ' OR ') . '';
		// find all components so we dont duplicate them
		$aExistComponents = $this->database()->select('component, module_id, product_id')
			->from(Phpfox::getT('component'))
			->where($sWhere)
			->execute('getSlaveRows');
		// find all blocks 
		$aExistBlocks = $this->database()->select('component, module_id, product_id')
			->from(Phpfox::getT('block'))
			->where($sWhere)
			->execute('getSlaveRows');
		
		$aExistsBlock = array();
		$aExistsComponent = array();
		$aToAddBlock = array();
		$aToAddComponent = array();
		// loop so we know which ones we dont need
		foreach ($aFields as $aField)
		{
		   $iExistsComponent = $this->database()->select('component_id')
			   ->from(Phpfox::getT('component'))
			   ->where('component = "cf_' . $aField['field_name'] . '"' 
				   . ' AND product_id = "' . $aField['product_id'] .'"'
				   . ' AND module_id = "' . $aField['module_id'] .'"')
			    ->execute('getSlaveField');
		   if ($iExistsComponent < 1)
		   {
		       $this->database()->insert(Phpfox::getT('component'),array(
			    'component' => 'cf_' . $aField['field_name'],
			    'm_connection' => null,
			    'module_id' =>  $aField['module_id'],
			    'product_id' => $aField['product_id'],
			    'is_controller' => '0',
			    'is_block' => '1',
			    'is_active' => '1'
			));
		   }
			 
		   $iExistsBlock = $this->database()->select('block_id')
			   ->from(Phpfox::getT('block'))
			   ->where('component = "cf_' . $aField['field_name'] . '"' 
				   . ' AND product_id = "' . $aField['product_id'] .'"'
				   . ' AND module_id = "custom"')
			    ->execute('getSlaveField');
		   
		    if ($iExistsBlock < 1)
		    {
			
			$this->database()->insert(Phpfox::getT('block'), array(
			    'title' => Phpfox::getPhrase($aField['phrase_var_name']),
			    'type_id' => '0',
			    'm_connection' => 'profile.info',
			    'module_id' => 'custom',//$aField['module_id'],
			    'product_id' => $aField['product_id'],
			    'component' => 'cf_' . $aField['field_name'],
			    'location' => '2',
			    'is_active' => '1',
			    'ordering' => '10',
			    'disallow_access' => null,
			    'can_move' => '1',
			    'version_id' => '1'
			));
		    }
		}
		
		$this->cache()->remove();
	    }
	}	
	
	/**
	 * The return of this function eventually goes to addComment
	 * @param type $iId
	 * @return string 
	 */
	public function getCommentItem($iId)
	{
		
		$aRow = $this->database()->select('field_id AS comment_item_id, f.user_id AS comment_user_id')
				->join(Phpfox::getT('feed'), 'f', 'f.item_id = cf.field_id')
				->from(Phpfox::getT('custom_field'), 'cf')
				->where('cf.field_id = ' . (int) $iId)
				->execute('getSlaveRow');

		$aRow['privacy_comment'] = '0';
		$aRow['comment_view_id'] = '0';

		if (!Phpfox::getService('comment')->canPostComment($aRow['comment_user_id'], $aRow['privacy_comment']))
		{
			Phpfox_Error::set('Unable to post a comment on this item due to privacy settings.');

			unset($aRow['comment_item_id']);
		}

		return $aRow;
	}

	
	public function addComment($aVals, $iUserId = null, $sUserName = null)
	{
		$aRow = $this->database()->select('cf.field_id,f.feed_id, cf.phrase_var_name, u.full_name, u.gender, u.user_id, u.user_name')
				->from(Phpfox::getT('custom_field'), 'cf')
				->join(Phpfox::getT('feed'), 'f', 'f.item_id = cf.field_id')
				->join(Phpfox::getT('user'), 'u', 'u.user_id = f.user_id')
				->where('cf.field_id = ' . (int) $aVals['item_id'])
				->execute('getSlaveRow');

		if (!isset($aRow['field_id']))
		{
			return Phpfox_Error::trigger('Invalid callback on comment.');
		}

		// Update the post counter if its not a comment put under moderation or if the person posting the comment is the owner of the item.
		//$this->database()->updateCounter('user_custom_data', 'total_comment', 'field_id', $aRow['field_id'] . ' AND user_id = ' . $aRow['user_id']);				
		$iCount = $this->database()->select('total_comment')
				->from(Phpfox::getT('user_custom_data'))
				->where('field_id = ' . (int) $aVals['item_id'] . ' AND user_id = ' . $aRow['user_id'])
				->execute('getSlaveField');

		$this->database()->update(Phpfox::getT('user_custom_data'), 
				array('total_comment' => ($iCount + 1)), 'field_id = ' . (int)$aRow['field_id'] . ' AND user_id = ' . $aRow['user_id']);

		if (Phpfox::isModule('feed'))
		{
			Phpfox::getService('feed.process')->add('comment_custom', $aVals['item_id']);
		}
		// Send the user an email
		$sLink = Phpfox::getLib('url')->makeUrl($aRow['user_name'], array('feed-id' => $aRow['feed_id']));//Phpfox::getLib('url')->permalink('custom', $aRow['field_id'], Phpfox::getPhrase($aRow['phrase_var_name']));

		Phpfox::getService('comment.process')->notify(array(
			'user_id' => $aRow['user_id'],
			'item_id' => $aRow['field_id'],
			'owner_subject' => Phpfox::getUserBy('full_name') . ' commented on your profile update "' . $this->preParse()->clean(Phpfox::getPhrase($aRow['phrase_var_name']), 100) . '".',
			'owner_message' => Phpfox::getUserBy('full_name') . " commented on your profile_update \"<a href=\"" . $sLink . "\">" . Phpfox::getPhrase($aRow['phrase_var_name']) . "</a>\".\nTo see the comment thread, follow the link below:\n<a href=\"" . $sLink . "\">" . $sLink . "</a>",
			'owner_notification' => 'comment.add_new_comment',
			'notify_id' => 'comment_custom',
			'mass_id' => 'custom',
			'mass_subject' => (Phpfox::getUserId() == $aRow['user_id'] ? Phpfox::getUserBy('full_name') . ' commented on ' . Phpfox::getService('user')->gender($aRow['gender'], 1) . ' profile update.' : Phpfox::getUserBy('full_name') . ' commented on ' . $aRow['full_name'] . '\'s video.'),
			'mass_message' => (Phpfox::getUserId() == $aRow['user_id'] ? Phpfox::getUserBy('full_name') . " commented on " . Phpfox::getService('user')->gender($aRow['gender'], 1) . " profile update \"<a href=\"" . $sLink . "\">" . Phpfox::getPhrase($aRow['phrase_var_name']) . "</a>\".\nTo see the comment thread, follow the link below:\n<a href=\"" . $sLink . "\">" . $sLink . "</a>" : Phpfox::getUserBy('full_name') . " commented on " . $aRow['full_name'] . "'s profile update \"<a href=\"" . $sLink . "\">" . Phpfox::getPhrase($aRow['phrase_var_name']) . "</a>\".\nTo see the comment thread, follow the link below:\n<a href=\"" . $sLink . "\">" . $sLink . "</a>")
				)
		);
	}

	public function deleteComment($iId)
	{

		//$this->database()->updateCounter('video', 'total_comment', 'video_id', $iId, true);
	}

	public function getCommentNotification($aNotification)
	{
		$aRow = $this->database()->select('cf.field_id, cf.phrase_var_name, f.feed_id, u.user_id, u.gender, u.user_name, u.full_name')
				->from(Phpfox::getT('custom_field'), 'cf')
				->join(Phpfox::getT('feed'), 'f', 'f.item_id = cf.field_id')
				->join(Phpfox::getT('user'), 'u', 'u.user_id = f.user_id')
				->where('cf.field_id = ' . (int) $aNotification['item_id'])
				->execute('getSlaveRow');

		$sPhrase = '';
		if ($aNotification['user_id'] == $aRow['user_id'] && !isset($aNotification['extra_users']))
		{
			$sPhrase = Phpfox::getService('notification')->getUsers($aNotification) . ' commented on ' . Phpfox::getService('user')->gender($aRow['gender'], 1) . ' profile update "' . Phpfox::getLib('parse.output')->shorten(Phpfox::getPhrase($aRow['phrase_var_name']), Phpfox::getParam('notification.total_notification_title_length'), '...') . '"';
		}
		elseif ($aRow['user_id'] == Phpfox::getUserId())
		{
			$sPhrase = Phpfox::getService('notification')->getUsers($aNotification) . ' commented on your profile update "' . Phpfox::getLib('parse.output')->shorten(Phpfox::getPhrase($aRow['phrase_var_name']), Phpfox::getParam('notification.total_notification_title_length'), '...') . '"';
		}
		else
		{
			$sPhrase = Phpfox::getService('notification')->getUsers($aNotification) . ' commented on <span class="drop_data_user">' . $aRow['full_name'] . '\'s</span> profile update "' . Phpfox::getLib('parse.output')->shorten(Phpfox::getPhrase($aRow['phrase_var_name']), Phpfox::getParam('notification.total_notification_title_length'), '...') . '"';
		}

		return array(
			'link' => Phpfox::getLib('url')->makeUrl($aRow['user_name'], array('feed-id' => $aRow['feed_id'])),
			'message' => $sPhrase,
			'icon' => Phpfox::getLib('template')->getStyle('image', 'activity.png', 'blog')
		);
	}

	/**
	 * If a call is made to an unknown method attempt to connect
	 * it to a specific plug-in with the same name thus allowing 
	 * plug-in developers the ability to extend classes.
	 *
	 * @param string $sMethod is the name of the method
	 * @param array $aArguments is the array of arguments of being passed
	 */
	public function __call($sMethod, $aArguments)
	{
		/**
		 * Check if such a plug-in exists and if it does call it.
		 */
		if ($sPlugin = Phpfox_Plugin::get('custom.service_callback__call'))
		{
			return eval($sPlugin);
		}

		/**
		 * No method or plug-in found we must throw a error.
		 */
		Phpfox_Error::trigger('Call to undefined method ' . __CLASS__ . '::' . $sMethod . '()', E_USER_ERROR);
	}
}

?>