<?php
/**
 * [PHPFOX_HEADER]
 */

defined('PHPFOX') or exit('NO DICE!');

/**
 * 
 * 
 * @copyright		[PHPFOX_COPYRIGHT]
 * @author  		Raymond Benc
 * @package  		Module_Group
 * @version 		$Id: index.class.php 2525 2011-04-13 18:03:20Z Raymond_Benc $
 */
class Group_Component_Controller_Index extends Phpfox_Component
{
	/**
	 * Class process method wnich is used to execute this component.
	 */
	public function process()
	{
		Phpfox::getUserParam('group.can_access_groups', true);
		define('PHPFOX_GROUP_VIEW', true);
		
		$this->template()->setHeader(array('<script type="text/javascript">var bIsGroup = true;</script>'));
		if ($this->request()->get('req2') && $this->request()->get('req2') != 'category')
		{
			return Phpfox::getLib('module')->setController('group.view');
		}
		
		if (($iDeleteId = $this->request()->getInt('delete')))
		{
			if (Phpfox::getService('group.process')->delete($iDeleteId))
			{
				$this->url()->send('group', null, Phpfox::getPhrase('group.group_successfully_deleted'));
			}
		}
		
		if (($iRedirect = $this->request()->getInt('redirect')) && ($sUrl = Phpfox::getService('group.callback')->getFeedRedirect($iRedirect)))
		{
			$this->url()->forward($sUrl);
		}		
		
		$oServiceGroupBrowse = Phpfox::getService('group.browse');
		$oDatabase = Phpfox::getLib('database');
		$oParseInput = Phpfox::getLib('parse.input');
		$iPage = $this->request()->get('page');
		$sView = $this->request()->get('view', false);
		$iPageSize = 20;
		$sCategory = null;	
		$iCategoryCount = 0;		
		
		$this->template()->setTitle(Phpfox::getPhrase('group.groups'))->setBreadcrumb(Phpfox::getPhrase('group.groups'), $this->url()->makeUrl('group'));
		
		$aRequests = $this->request()->getRequests();			
		foreach ($aRequests as $sKey => $sValue)
		{			
			if (!preg_match("/req[0-9]/", $sKey))
			{
				continue;
			}
			
			$iCategoryCount++;
			
			if ($iCategoryCount < 3)
			{
				continue;
			}
			
			$sCategory = $sValue;
		}		
		
		$this->setParam('sCategory', $sCategory);
		
		$aPages = array(12, 15, 18, 21);
		$aDisplays = array();
		foreach ($aPages as $iPageCnt)
		{
			$aDisplays[$iPageCnt] = Phpfox::getPhrase('core.per_page', array('total' => $iPageCnt));
		}		
	
		$aFilters = array(
			'display' => array(
				'type' => 'select',
				'options' => $aDisplays,
				'default' => '12'
			),
			'country' => array(
				'type' => 'select',
				'options' => Phpfox::getService('core.country')->get(),
				'search' => 'AND m.country_iso = \'[VALUE]\'',
				'style' => 'width:90%;',
				'id' => 'country_iso',
				'add_any' => true
			),
			'country_child_id' => array(
				'type' => 'select',
				'search' => 'AND m.country_child_id = \'[VALUE]\'',
				'clone' => true
			),
			'sort' => array(
				'type' => 'select',
				'options' => array(
					'time_stamp' => Phpfox::getPhrase('group.date_added'),
					'total_member' => Phpfox::getPhrase('group.most_popular'),
					'title' => Phpfox::getPhrase('group.name')
				),
				'default' => 'time_stamp',
				'alias' => 'm'
			),
			'sort_by' => array(
				'type' => 'select',
				'options' => array(
					'DESC' => Phpfox::getPhrase('core.descending'),
					'ASC' => Phpfox::getPhrase('core.ascending')
				),
				'default' => 'DESC'
			),
			'keyword' => array(
				'type' => 'input:text',
				'size' => 20,
				'search' => 'AND (m.title LIKE \'%[VALUE]%\' OR m.short_description LIKE \'%[VALUE]\')'
			),
			'city' => array(
				'type' => 'input:text',
				'size' => 20,
				'search' => 'AND m.city LIKE \'%[VALUE]%\''
			),
			'zip' => array(
				'type' => 'input:text',
				'size' => 10,
				'search' => 'AND m.postal_code = \'[VALUE]\''
			)
		);
		
		$oFilter = Phpfox::getLib('search')
			->set(array(
				'type' => 'browse',
				'filters' => $aFilters,
				'search' => 'keyword'
			)
		);		
		
		if (!Phpfox::getUserParam('group.can_approve_groups'))
		{
			$oFilter->setCondition('AND m.is_public = 0');
		}
		
		if ($sView)
		{
			Phpfox::isUser(true);
			
			switch ($sView)
			{
				case 'owner':
					if ($iUserId = $this->request()->getInt('id'))
					{
						$aUser = Phpfox::getService('user')->getUser($iUserId, Phpfox::getUserField());
						if (isset($aUser['user_id']))
						{
							$oFilter->setCondition('AND m.user_id = ' . $iUserId . '');
							
							$this->template()->setBreadcrumb(Phpfox::getLib('parse.output')->clean($aUser['full_name']), $this->url()->makeUrl($aUser['user_name']), true);
						}
					}
					break;
				case 'my':
					$oFilter->setCondition('AND m.user_id = ' . Phpfox::getUserId() . '');
					$this->template()->setBreadcrumb(Phpfox::getPhrase('group.groups_i_ve_created'), null, true);
					break;
				case 'member':
					$oServiceGroupBrowse->member(1);
					$this->template()->setBreadcrumb(Phpfox::getPhrase('group.all_my_group_memberships'), null, true);
					break;
				case 'invite':
					$oServiceGroupBrowse->member(0);
					$this->template()->setBreadcrumb(Phpfox::getPhrase('group.my_invites'), null, true);
					break;		
				case 'approval':
					if (Phpfox::getUserParam('group.can_approve_groups'))
					{
						$oFilter->setCondition('AND m.is_public = 1');
						$this->template()->setBreadcrumb(Phpfox::getPhrase('group.pending_approval'), null, true);
					}					
					break;
				default:
					if (!Phpfox::getUserParam('group.can_view_secret_group'))
					{
						$oFilter->setCondition('AND m.view_id = 0');		
					}
					break;		
			}
		}
		else 
		{			
			if ($iUserId = $this->request()->getInt('user'))
			{
				$aUser = Phpfox::getService('user')->getUser($iUserId, Phpfox::getUserField());
				if (isset($aUser['user_id']))
				{
					$oServiceGroupBrowse->member(1);
					$oServiceGroupBrowse->user($aUser['user_id']);
					
					$this->template()->setBreadcrumb(Phpfox::getLib('parse.output')->clean($aUser['full_name']), $this->url()->makeUrl($aUser['user_name']), true);
				}
			}
			else 
			{
				if (!Phpfox::getUserParam('group.can_view_secret_group'))
				{
					$oFilter->setCondition('AND m.view_id IN(0,1)');
				}
			}
		}			

		if ($this->request()->getInt('sponsor') == 1)
		{
		    $oFilter->setCondition('AND m.is_sponsor != 1');
		    Phpfox::addMessage(Phpfox::getPhrase('group.sponsor_help'));
		}		
		
		$oServiceGroupBrowse->condition($oFilter->getConditions())
			->category($sCategory)		
			->order($oFilter->getSort())	
			->page($iPage)
			->size($oFilter->getDisplay())
			->execute();		
			
		Phpfox::getLib('pager')->set(array('page' => $iPage, 'size' => $oFilter->getDisplay(), 'count' => $oFilter->getSearchTotal($oServiceGroupBrowse->getCount())));		
		
		$this->template()->setHeader('cache', array(
					'pager.css' => 'style_css',
					'country.js' => 'module_core'					
				)
			)
			->assign(array(
					'aGroups' => $oServiceGroupBrowse->get(),
					'bIsSearch' => $oFilter->isSearch(),
					'sGroupView' => $sView
				)
			);		
			
		if ($sCategory !== null)
		{
			$aCategories = Phpfox::getService('group.category')->getParentBreadcrumb($sCategory);			
			$iCnt = 0;
			foreach ($aCategories as $aCategory)
			{
				$iCnt++;
				
				$this->template()->setTitle($aCategory[0]);					
				$this->template()->setBreadcrumb($aCategory[0], $aCategory[1], ($iCnt === count($aCategories) ? true : false));
			}			
		}			
	}
	
	/**
	 * Garbage collector. Is executed after this class has completed
	 * its job and the template has also been displayed.
	 */
	public function clean()
	{
		(($sPlugin = Phpfox_Plugin::get('group.component_controller_index_clean')) ? eval($sPlugin) : false);
	}
}

?>